'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.buildConfig = buildConfig;
exports.install = install;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _knex = require('knex');

var _knex2 = _interopRequireDefault(_knex);

var _override = require('./override');

var debug = require('./debug')('client-multi-tenant');

/**
 * Build knex tenant configuration changing details related to multitenant
 */

function buildConfig(config, tenantId) {
  var multitenantConfig = _extends({}, config || {}, {
    tenantId: tenantId
  });

  multitenantConfig.migrations = _extends({}, multitenantConfig.migrations || {}, {
    // custom migration with the table name prefix
    tableName: tenantId + '_' + (multitenantConfig.migrations.tableName || 'knex_migrations')
  });

  return multitenantConfig;
}

/**
 * Installs the tenant monkey patch on knex.
 *
 * It overrides some base knex functions changing the original behavior to
 * include multitenant configurations such as the tenant prefix in every table.
 */

function install() {
  Object.defineProperty(_knex2['default'].Client.prototype, 'tenantId', {
    get: function get() {
      return this.config.tenantId;
    }
  });

  (0, _override.override)(_knex2['default'].Client.prototype.QueryBuilder.prototype, 'toSQL', (0, _override.after)(function (sql) {
    debug('knex.Client.prototype.QueryBuilder.prototype.toSQL', arguments);
    sql.sql = applyTenant(sql.sql, this.client.tenantId);
    return sql;
  }));

  (0, _override.override)(_knex2['default'].Client.prototype.Raw.prototype, 'set', (0, _override.before)(function (sql, bindings) {
    debug('knex.Client.prototype.Raw.prototype.set', arguments);
    var tenantSQL = applyTenant(sql, this.client.tenantId);
    return [tenantSQL, bindings];
  }));

  (0, _override.override)(_knex2['default'].Client.prototype.SchemaBuilder.prototype, 'toSQL', (0, _override.after)(function (sql) {
    var _this = this;

    debug('knex.Client.prototype.SchemaBuilder.prototype.toSQL', arguments);
    return sql.map(function (q) {
      q.sql = applyTenant(q.sql, _this.client.tenantId);
      return q;
    });
  }));

  (0, _override.override)(_knex2['default'].Client.prototype.Runner.prototype, 'query', (0, _override.after)(function callee$1$0(promise, originalArgs) {
    var options,
        result,
        args$2$0 = arguments;
    return regeneratorRuntime.async(function callee$1$0$(context$2$0) {
      var _this2 = this;

      while (1) switch (context$2$0.prev = context$2$0.next) {
        case 0:
          debug('knex.Client.prototype.Runner.prototype.query', args$2$0);
          options = originalArgs[0].options;
          context$2$0.next = 4;
          return regeneratorRuntime.awrap(promise);

        case 4:
          result = context$2$0.sent;

          if (!(!options || !options.nestTables)) {
            context$2$0.next = 7;
            break;
          }

          return context$2$0.abrupt('return', result);

        case 7:
          return context$2$0.abrupt('return', result.map(function (row) {
            return Object.keys(row).reduce(function (processedRow, tableJoinName) {
              processedRow[unapplyTenant(tableJoinName, _this2.client.tenantId)] = row[tableJoinName];
              return processedRow;
            }, {});
          }));

        case 8:
        case 'end':
          return context$2$0.stop();
      }
    }, null, this);
  }));
}

function unapplyTenant(sql, tenant) {
  var regexp = new RegExp('^(' + tenant + '_)');
  return sql.replace(regexp, '$_');
}

function applyTenant(sql, tenant) {
  return sql.replace(/\$_/g, tenant + '_');
}