'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj['default'] = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _knex = require('knex');

var _knex2 = _interopRequireDefault(_knex);

var _promiseAsyncCache = require('promise-async-cache');

var _promiseAsyncCache2 = _interopRequireDefault(_promiseAsyncCache);

var _knexTenantSupport = require('./knex-tenant-support');

var knexTenantSupport = _interopRequireWildcard(_knexTenantSupport);

var debug = require('./debug')('tenant');

/**
 * Defines whenever the tenant monkey patch on knex has been installed
 */
var isMultiTenantSupportInstalled = false;

/**
 * Keep knex tenants in memory and handle race conditions
 */
var cache = new _promiseAsyncCache2['default']({
  /**
   * Build knex tenant and load it to the cache
   */
  load: function load(tenantId, baseKnex) {
    var proxyKnex;
    return regeneratorRuntime.async(function load$(context$1$0) {
      while (1) switch (context$1$0.prev = context$1$0.next) {
        case 0:
          if (isMultiTenantSupportInstalled) {
            context$1$0.next = 11;
            break;
          }

          context$1$0.prev = 1;

          debug('installing knextancy');
          knexTenantSupport.install();
          isMultiTenantSupportInstalled = true;
          context$1$0.next = 11;
          break;

        case 7:
          context$1$0.prev = 7;
          context$1$0.t0 = context$1$0['catch'](1);

          debug('Error installing knex multi tenant support', context$1$0.t0.stack || context$1$0.t0);
          throw context$1$0.t0;

        case 11:

          debug('building knex for new tenant %d', tenantId);
          proxyKnex = (0, _knex2['default'])(knexTenantSupport.buildConfig(baseKnex.client.config, tenantId));

          Object.defineProperty(proxyKnex, 'tenantId', {
            get: function get() {
              return this.client.tenantId;
            }
          });

          debug('initializing multi tenant database');
          debug('running migration tasks');
          context$1$0.next = 18;
          return regeneratorRuntime.awrap(proxyKnex.migrate.latest());

        case 18:

          debug('running seed tasks');
          context$1$0.next = 21;
          return regeneratorRuntime.awrap(proxyKnex.seed.run());

        case 21:
          return context$1$0.abrupt('return', proxyKnex);

        case 22:
        case 'end':
          return context$1$0.stop();
      }
    }, null, this, [[1, 7]]);
  }
});

exports['default'] = function callee$0$0(baseKnex, tenantId) {
  return regeneratorRuntime.async(function callee$0$0$(context$1$0) {
    while (1) switch (context$1$0.prev = context$1$0.next) {
      case 0:
        context$1$0.prev = 0;
        return context$1$0.abrupt('return', cache.get(tenantId, baseKnex));

      case 4:
        context$1$0.prev = 4;
        context$1$0.t0 = context$1$0['catch'](0);

        debug('Error on initializing the multi tenant database', context$1$0.t0.stack || context$1$0.t0);
        throw context$1$0.t0;

      case 8:
      case 'end':
        return context$1$0.stop();
    }
  }, null, this, [[0, 4]]);
};

module.exports = exports['default'];